#ifndef _STRINGTOKENIZER_H_
#define _STRINGTOKENIZER_H_

#include <stdio.h>
#include <string.h>

class StringTokenizer
{

public:
  StringTokenizer(char *sData, char *sDelimiter)
  {
    data = sData;
    dataLen = static_cast<int>(strlen(data));
    delimiter = sDelimiter;
    delimiterLen = static_cast<int>(strlen(sDelimiter));
    stopchar = '\0';
    offset = 0;
    done = false;
  }

  virtual ~StringTokenizer(){}    

char* nextToken()
  {
    if(done)
    {
        token[0] = '\0';
        return token;
    }

    tokenSize = 0;
    token[0] = '\0';
    for(int i = 0; offset < dataLen; i++) 
    {
      token[i] = data[offset];
      token[i+1] = '\0'; //Forces string end. Keeps non-ending strings from ever happening.

      for(int d = 0; d < delimiterLen; d++)
      {
        if(token[i] == delimiter[d])
        {
          // Is the delimiter the first char in the token?
          if(i == 0)
          {
              if(offset == dataLen)
              {
                  done = true;
                  return NULL;
              }
              //offset++;
              //continue;
              --i;
              break;
          }

          token[i] = '\0';
          if(offset == dataLen)
            done = true;
          offset++;
          tokenSize = i;
          return token;
        }else
        if(token[i] == stopchar)
        {
            token[i] = '\0';
            done = true;
            return token;
        }
      }

      offset++;
    }
    done = true;
    return token;
  }

bool hasMoreTokens() const
  {
    return !done;
  }

int getDataLength() const
  {
    return dataLen;
  }

int getTokenSize() const
  {
    return tokenSize;
  }

int getNumTokens()
  {
    int temp = offset;
    int num = 0;
		bool doneswap = done;	

    while(hasMoreTokens())
    {
      if (nextToken() != NULL)
      {
        ++num;
      }
    }
		
		done = doneswap; //reset this to it's previous state
    
    offset = temp;
    return num;
  }

void reset()
{
  offset = 0;
}

// This can be dangerous. If the char* goes out of scope, 
// this could point to invalid memory.
void SetDelimiter(char* sDelimiter)
{
    delimiter = sDelimiter;
}

// This sets a char that will stop the current parse
// and return a token up to that point
void SetStopChar(char newStopChar)
{
    stopchar = newStopChar;
}


private:
  char *data;
  char *delimiter;
  char stopchar;
  int offset;
  bool done;
  int dataLen;
  int delimiterLen;
  int tokenSize;
  char token[512];

};

#endif
