// CaddyVM.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <stdio.h>
#include "commondefs.h"
#include "caddycpu.h"

const char* appname = "CaddyVM";
const char* version = "1.0";

int main(int argc, char** argv)
{
    printf("---------------------------\n");
    printf("Caddy Virtual Machine v%s\n", version);
    printf("---------------------------\n");
    printf("September 2003\n\n");

    if(argc < 2)
    {
        printf("Invalid argument count!\n\n");
        printf("Usage: %s <filename>\n\n", appname);
        return 1;
    }

    CaddyCPU* pCPU = new CaddyCPU;
    if(NULL == pCPU)
    {
        return 1;
    }

    //Do something interesting...
    //Like read in a program, or simply assign one
    
    // ***********************************************************
    // Start Tiny test program
    // ***********************
    /*
      // Give the B register some data to move around
    pCPU->SetMemoryAddress(0x00, 0x06); // MVI B, 0xFF
    pCPU->SetMemoryAddress(0x01, 0xFF);

      // Move the cotnents of reg B into reg A
    pCPU->SetMemoryAddress(0x02, 0x78); // Mov A, B

      // Move reg A contents to memory address 0x0F
    pCPU->SetMemoryAddress(0x03, 0x26); //MVI H, 0x00
    pCPU->SetMemoryAddress(0x04, 0x00); 
    pCPU->SetMemoryAddress(0x05, 0x2E); //MVI L, 0x2F
    pCPU->SetMemoryAddress(0x06, 0x2F); 
    pCPU->SetMemoryAddress(0x07, 0x77); // Mov 0x002F, A

      // Move contents memory address 0x0F to reg C
    pCPU->SetMemoryAddress(0x08, 0x26); //MVI H, 0x00
    pCPU->SetMemoryAddress(0x09, 0x00); 
    pCPU->SetMemoryAddress(0x0A, 0x2E); //MVI L, 0x2F
    pCPU->SetMemoryAddress(0x0B, 0x2F); 
    pCPU->SetMemoryAddress(0x0C, 0x4E); // Mov C, 0x002F

      // Add Reg C to Reg A
    pCPU->SetMemoryAddress(0x0D, 0x81); // Add C

      // Add mem address to Reg A
    pCPU->SetMemoryAddress(0x0E, 0x26); //MVI H, 0x00
    pCPU->SetMemoryAddress(0x0F, 0x00); 
    pCPU->SetMemoryAddress(0x10, 0x2E); //MVI L, 0x2F
    pCPU->SetMemoryAddress(0x11, 0x2F); 
    pCPU->SetMemoryAddress(0x12, 0x86); // Add 0x002F

      // MVI to memory
    pCPU->SetMemoryAddress(0x13, 0x26); //MVI H, 0x00
    pCPU->SetMemoryAddress(0x14, 0x00); 
    pCPU->SetMemoryAddress(0x15, 0x2E); //MVI L, 0x30
    pCPU->SetMemoryAddress(0x16, 0x30); 
    pCPU->SetMemoryAddress(0x17, 0x36); // MVI 0x0030, 0xAB
    pCPU->SetMemoryAddress(0x18, 0xAB);

      // Set program end
    pCPU->SetMemoryAddress(0x19, 0xFF); // end it
    */
    // ***********************
    // End Tiny test program
    // ***********************************************************

    //Read in our program test.bin for now.
    {
    printf("Opening test.bin\n\n");

    FILE* fp;
    fp = fopen(argv[1], "rb");
    if(fp)
    {
        for(int i = 0; (!ferror(fp)) && (!feof(fp)); i++)
        {
            uint8 block; 
            int numRead = (int)fread(&block, 1, 1, fp);

            if(0 != numRead)
            {
                pCPU->SetMemoryAddress(i, block);
            }
            else
            {
                break;
            }
        }

    }else
    {
        printf("Error! Cannot open %s\n", argv[1]);
        return 1;
    }
    }


    // Reset the program counter (instruction pointer
    pCPU->SetProgramCounter(0x00);

    printf("Executing %s\n", argv[1]);

    // Ok, run the program that we have stored in "memory"
    RESULT res = RESULT_OK;
    for(int i = 0; res != RESULT_DONE; i++)
    {
        // it's ticking...
        res = pCPU->Tick();
    }

    // Now that the tiny program is done, write out a state dump:
    printf("Writing output log\n");

    FILE* fp;
    fp = fopen("CaddyVMStateMap.txt", "w");
    if(fp)
    {

        fprintf(fp, "This output file was produced by CaddyVM.\n");
        fprintf(fp, "It represents the state of the VM after\n");
        fprintf(fp, "running an application.\n\n");

        fprintf(fp, "Output log from: %s\n\n", argv[1]);

        // Write info header
        fprintf(fp, "CaddyVM (Post Run) VM CPU State Map\n");
        fprintf(fp, "\n");

        // Get the registers:
        RegisterSet Reg = pCPU->GetRegisters();
        fprintf(fp, "Registers\n");
        fprintf(fp, "Reg A: %x (%d)\n", Reg.a, Reg.a);
        fprintf(fp, "Reg B: %x (%d)\n", Reg.b, Reg.b);
        fprintf(fp, "Reg C: %x (%d)\n", Reg.c, Reg.c);
        fprintf(fp, "Reg D: %x (%d)\n", Reg.d, Reg.d);
        fprintf(fp, "Reg E: %x (%d)\n", Reg.e, Reg.e);
        fprintf(fp, "Reg H: %x (%d)\n", Reg.h, Reg.h);
        fprintf(fp, "Reg L: %x (%d)\n", Reg.l, Reg.l);

        fprintf(fp, "\n");

        // Get the Flags
        FlagSet Flags = pCPU->GetFlags();
        fprintf(fp, "Flags\n");
        fprintf(fp, "S Z - AC - P - CY\n");
        fprintf(fp, "%d %d   %d    %d   %d\n",
            Flags.sign, 
            Flags.zero,
            Flags.ac,
            Flags.parity,
            Flags.carry
            );

        fprintf(fp, "\n");

        // Get Program counter/Instruction pointer
        fprintf(fp, "Program Counter/Instruction Pointer\n");
        fprintf(fp, "0x%x (%d)\n", pCPU->GetProgramCounter(), pCPU->GetProgramCounter());

        fprintf(fp, "\n");

        // Get Stack pointer
        fprintf(fp, "Stack Pointer\n");
        fprintf(fp, "0x%x (%d)\n", pCPU->GetStackPointer(), pCPU->GetStackPointer());

        fprintf(fp, "\n");

        // Get the memory map
        fprintf(fp, "Memory Map\n");
        for(int j = 0; j < MEMORY_SIZE; j++)
        {
            fprintf(fp, "0x%x: 0x%x (%d)\n", j,pCPU->GetMemoryAddress(j), pCPU->GetMemoryAddress(j));
        }

        fprintf(fp, "\n");

        fclose(fp);
    }



    if(pCPU)
    {
        delete pCPU;
        pCPU = NULL;
    }
    printf("%s runtime complete.. exiting.\n\n", appname);

	return 0;
}

