#ifndef _CADDYCPU_H_
#define _CADDYCPU_H_

#include "commondefs.h"

#define MEMORY_SIZE 1024*64

// Register access
#define REG_B 0x00
#define REG_C 0x01
#define REG_D 0x02
#define REG_E 0x03
#define REG_H 0x04
#define REG_L 0x05
#define REG_NULL 0x06
#define REG_A 0x07

// Flag access
#define  FLAG_CARRY 0
#define  FLAG_PARITY 2
#define  FLAG_AC 4
#define  FLAG_ZERO 6
#define  FLAG_SIGN 7

enum EResultCodes
{
    RESULT_OK       = 0,
    RESULT_ERROR    = 1,
    RESULT_DONE     = 2
};
typedef EResultCodes RESULT;


struct RegisterSet
{
    uint8 b;
    uint8 c;
    uint8 d;
    uint8 e;
    uint8 h;
    uint8 l;
    uint8 pad; //This does nothing but help increase the access speed.
    uint8 a;
};

struct FlagSet
{
    bool carry;
    bool pad;
    bool parity;
    bool pad2;
    bool ac;
    bool pad3;
    bool zero;
    bool sign;
};

class CaddyCPU
{
public:
    CaddyCPU();
    ~CaddyCPU();

    RESULT Clear();
    RESULT Tick(); //Cycle once
    RESULT SetMemoryAddress(uint16 iAddress, uint8 iValue);
    const uint8&  GetMemoryAddress(uint16 iAddress) const;
    RESULT SetRegister(uint8 iRegister, uint8 iValue);
    const  uint8& GetRegister(uint8 iReg) const;
    RESULT SetProgramCounter(uint16 iAddress);
    uint16 GetProgramCounter(){return m_iProgramCounter;}
    uint16 GetStackPointer(){return m_iStackPointer;}
    const RegisterSet& GetRegisters() const { return m_Reg; }
    const FlagSet& GetFlags() const { return m_Flags; }

protected:

    RESULT      ExecuteInstruction();

    RegisterSet m_Reg;
    FlagSet     m_Flags;
    uint16      m_iProgramCounter;
    uint16      m_iStackPointer;    
    uint8       m_Memory[MEMORY_SIZE]; //64k

    //Instructions/Opcode functions
    RESULT      OPCODE_MVI();
    RESULT      OPCODE_MOV();
    RESULT      OPCODE_ADD();
    RESULT      OPCODE_CMP();
    RESULT      OPCODE_RET();

};

#endif